import os
import copy
import unittest
from orangecontrib.text.corpus import Corpus


DATASET_PATH = os.path.join(os.path.dirname(__file__), '..', 'datasets')


class CorpusTests(unittest.TestCase):
    def test_corpus_from_file(self):
        c = Corpus.from_file(os.path.join(DATASET_PATH, 'bookexcerpts.tab'))
        self.assertEqual(len(c), 140)
        self.assertEqual(len(c.domain), 1)
        self.assertEqual(len(c.domain.metas), 1)
        self.assertEqual(c.metas.shape, (140, 1))

    def test_corpus_from_file_just_text(self):
        c = Corpus.from_file(os.path.join(DATASET_PATH, 'deerwester.tab'))

        self.assertEqual(len(c), 9)
        self.assertEqual(len(c.domain), 0)
        self.assertEqual(len(c.domain.metas), 1)
        self.assertEqual(c.metas.shape, (9, 1))

    def test_extend_corpus(self):
        c = Corpus.from_file(os.path.join(DATASET_PATH, 'bookexcerpts.tab'))
        n_classes = len(c.domain.class_var.values)
        c_copy = c.copy()
        new_y = [c.domain.class_var.values[int(i)] for i in c.Y]
        new_y[0] = 'teenager'
        c.extend_corpus(c.documents, c.metas, new_y)

        self.assertEqual(len(c), len(c_copy)*2)
        self.assertEqual(c.Y.shape[0], c_copy.Y.shape[0]*2)
        self.assertEqual(c.metas.shape[0], c_copy.metas.shape[0]*2)
        self.assertEqual(c.metas.shape[1], c_copy.metas.shape[1])
        self.assertEqual(len(c_copy.domain.class_var.values), n_classes+1)



import numpy as np
from gensim import corpora, models, matutils

from Orange.data.table import Table
from Orange.data.domain import Domain, ContinuousVariable, StringVariable
from orangecontrib.text.topics import Topics

def chunk_list(l, num):
    num = min(len(l), num)
    avg = len(l) / float(num)
    out = []
    last = 0.0
    while last < len(l):
        out.append(l[int(last):int(last + avg)])
        last += avg
    return out

MAX_WORDS = 1000


class LDA:
    def __init__(self, text, num_topics=5, callback=None):
        """
        Wraper for Gensim LDA model.

        :param text: Preprocessed text.
        :param num_topics: Number of topics to infer.
        :return: None
        """
        self.text = text
        self.num_topics = num_topics

        # generate dict and corpus
        dictionary = corpora.Dictionary(self.text)
        corpus = [dictionary.doc2bow(t) for t in self.text]

        lda = models.LdaModel(id2word=dictionary, num_topics=self.num_topics)
        done = 0
        for i, part in enumerate(chunk_list(corpus, 95)):
            lda.update(part)
            done += len(part)
            callback(95.0*done/len(corpus))

        corpus = lda[corpus]
        topics = lda.show_topics(num_topics=-1, num_words=3, formatted=False)
        names = [', '.join([i[1] for i in t]) for t in topics]
        names = ['Topic{} ({})'.format(i, n) for i, n in enumerate(names, 1)]

        self.topic_names = names
        self.corpus = corpus
        self.lda = lda

    def insert_topics_into_corpus(self, corp_in):
        """
        Insert topical representation into corpus.

        :param corp_in: Corpus into whic we want to insert topical representations
        :return: `Orange.data.table.Table`
        """
        matrix = matutils.corpus2dense(self.corpus,
                                       num_terms=self.num_topics).T

        # Generate the new table.
        attr = [ContinuousVariable(n) for n in self.topic_names]
        domain = Domain(attr,
                        corp_in.domain.class_vars,
                        metas=corp_in.domain.metas)

        return Table.from_numpy(domain,
                                matrix,
                                Y=corp_in._Y,
                                metas=corp_in.metas)

    def get_topics_table_by_id(self, topic_id):
        """
        Transform topics from gensim LDA model to table.

        :param lda: gensim LDA model.
        :return: `Orange.data.table.Table`.
        """
        topics = self.lda.show_topics(num_topics=-1, num_words=MAX_WORDS, formatted=False)
        if topic_id >= len(topics):
            raise ValueError("Too large topic ID.")

        num_words = max([len(it) for it in topics])

        data = np.zeros((num_words, 2), dtype=object)
        data[:, 0] = [item[1] for item in topics[topic_id]]
        data[:, 1] = [item[0] for item in topics[topic_id]]

        metas = [StringVariable(self.topic_names[topic_id]),
                 ContinuousVariable("Topic{}_weights".format(topic_id+1))]
        metas[-1]._out_format = '%.2e'

        domain = Domain([], metas=metas)
        t = Topics.from_numpy(domain,
                              X=np.zeros((num_words, 0)),
                              metas=data)
        t.W = data[:, 1]
        return t

    def get_top_words_by_id(self, topic_id):
        topics = self.lda.show_topics(num_topics=-1, num_words=10, formatted=False)
        if topic_id >= len(topics):
            raise ValueError("Too large topic ID.")
        return [item[1] for item in topics[topic_id]]




import numpy as np
from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer

from Orange.widgets.widget import OWWidget
from Orange.widgets.settings import Setting
from Orange.widgets import gui
from Orange.data import Table, DiscreteVariable, ContinuousVariable, Domain, StringVariable
from orangecontrib.text.corpus import Corpus
from orangecontrib.text.preprocess import Preprocessor


class Output:
    DATA = "Data"


class OWBagOfWords(OWWidget):
    # Basic widget info
    name = "Bag of Words"
    description = "Generates a bag of words from the input corpus."
    icon = "icons/BagOfWords.svg"
    priority = 40

    # Input/output
    inputs = [("Corpus", Corpus, "set_corpus"),
              ("Preprocessor", Preprocessor, "set_preprocessor")]
    outputs = [(Output.DATA, Table)]
    want_main_area = False

    _normalization_options = ['(none)', 'L1 (sum of elements)', 'L2 (Euclidean)']

    # Settings
    use_tfidf = Setting(False)
    normalization_type = Setting(0)

    def __init__(self):
        super().__init__()

        self.corpus = None
        self.preprocessor = None
        self.normalization = None

        # Pre-processing info.
        pp_info_box = gui.widgetBox(self.controlArea, "Pre-processing info")

        pp_info = "Includes punctuation: {}\n" \
                  "Lowercase: {}\n" \
                  "Transformation: {}\n" \
                  "Stop words removed: {}\n" \
                  "TF-IDF performed: {}\n" \
                  "Normalization: {}"\
            .format(False, False, None, None,
                    self.use_tfidf, self.normalization)
        self.pp_info_label = gui.label(pp_info_box, self, pp_info)

        # TF-IDF.
        tfidf_box = gui.widgetBox(self.controlArea, "TF-IDF", addSpace=False)

        self.tfidf_chbox = gui.checkBox(tfidf_box, self, "use_tfidf", "Use TF-IDF")
        self.tfidf_chbox.stateChanged.connect(self._tfidf_changed)
        ibox = gui.indentedBox(tfidf_box)
        self.norm_combo = gui.comboBox(ibox, self, 'normalization_type',
                                       items=self._normalization_options,
                                       label="Normalization:")
        self.norm_combo.activated[int].connect(self._select_normalization)
        self.norm_combo.setEnabled(self.use_tfidf)

        gui.button(self.controlArea, self, "&Apply", callback=self.apply, default=True)

    def set_preprocessor(self, data):
        self.preprocessor = data

    def set_corpus(self, data):
        self.corpus = data
        self.apply()

    def apply(self):
        # TODO Move the logic to the scripting module.
        new_table = None
        if self.corpus:
            if self.preprocessor:
                if self.use_tfidf:
                    cv = TfidfVectorizer(lowercase=self.preprocessor.lowercase,
                                         stop_words=self.preprocessor.stop_words,
                                         preprocessor=self.preprocessor.transformation,
                                         norm=self.normalization)
                else:
                    cv = CountVectorizer(lowercase=self.preprocessor.lowercase,
                                         stop_words=self.preprocessor.stop_words,
                                         preprocessor=self.preprocessor.transformation)

                pp_info_tag = "Includes punctuation: {}\nLowercase: {}\n" \
                              "Transformation: {}\n" \
                              "Stop words removed: {}\n" \
                              "TF-IDF performed: {}\n" \
                              "Normalization: {}"\
                    .format(self.preprocessor.incl_punct, self.preprocessor.lowercase,
                            self.preprocessor.transformation.name,
                            self.preprocessor.stop_words, self.use_tfidf,
                            self._normalization_options[self.normalization_type])
                self.pp_info_label.setText(pp_info_tag)
            else:
                cv = CountVectorizer(lowercase=True,
                                     binary=True)

            documents = self.corpus.documents
            feats = cv.fit(documents)  # Features.
            freqs = feats.transform(documents).toarray()  # Frequencies.

            # Generate the domain attributes.
            attr = [ContinuousVariable(f) for f in feats.get_feature_names()]

            # Construct a new domain.
            domain = Domain(attr, self.corpus.domain.class_vars, metas=self.corpus.domain.metas)

            # Create the table.
            new_table = Table.from_numpy(domain, freqs, Y=self.corpus._Y, metas=self.corpus.metas)
        self.send("Data", new_table)

    def _select_normalization(self, n):
        if n > 0:
            self.normalization = self._normalization_options[n].split(" ")[0].lower()
        else:
            self.normalization = None

    def _tfidf_changed(self):
        self.norm_combo.setEnabled(self.use_tfidf)



import os
import re
import math
import numpy as np
from time import sleep
from PyQt4.QtGui import *
from PyQt4 import QtCore, QtGui
from datetime import date, datetime
from urllib.error import HTTPError, URLError

from Orange.widgets.settings import Setting
from Orange.widgets.widget import OWWidget
from Orange.widgets import gui
from orangecontrib.text.corpus import Corpus
from orangecontrib.text.nyt import NYT, _parse_record_json, NYT_TEXT_FIELDS, _generate_corpus


def _i(name, icon_path="icons"):
    widget_path = os.path.dirname(os.path.abspath(__file__))
    return os.path.join(widget_path, icon_path, name)

class Output:
    CORPUS = "Corpus"

class OWNYT(OWWidget):
    name = "NY Times"
    description = "Load data from the New York Times article search API."
    icon = "icons/NYTimes.svg"
    priority = 20

    outputs = [(Output.CORPUS, Corpus)]
    want_main_area = False

    # Settings.
    recent_queries = Setting([])
    recent_api_keys = Setting([])
    year_from = Setting("")
    year_to = Setting("")
    # Text includes checkboxes.
    includes_headline = Setting(True)
    includes_lead_paragraph = Setting(True)
    includes_snippet = Setting(False)
    includes_abstract = Setting(False)
    includes_keywords = Setting(False)

    def __init__(self):
        super().__init__()

        self.output_corpus = None
        self.all_hits = 0
        self.num_retrieved = 0
        self.nyt_api = None
        self.api_key = ""
        self.api_key_is_valid = False
        self.query_running = False

        # To hold all the controls. Makes access easier.
        self.nyt_controls = []

        # Root box.
        parameter_box = gui.widgetBox(self.controlArea, addSpace=True)

        # API key box.
        api_key_box = gui.widgetBox(parameter_box, orientation=0)
        # Valid API key feedback.
        self.api_key_valid_label = gui.label(api_key_box, self, "")
        self.update_validity_icon()
        self.api_key_valid_label.setMaximumSize(self.api_key_valid_label.sizeHint())
        # Set API key button.
        self.open_set_api_key_dialog_button = gui.button(api_key_box, self, 'Article API key',
                                                         callback=self.open_set_api_key_dialog,
                                                         tooltip="Set the API key for this widget.")
        self.open_set_api_key_dialog_button.setFocusPolicy(QtCore.Qt.NoFocus)

        # Query box.
        query_box = gui.widgetBox(parameter_box, orientation=0)
        q_label = gui.label(query_box, self, "Query:")
        q_label.setMaximumSize(q_label.sizeHint())
        # Drop-down for recent queries.
        self.query_combo = QComboBox(query_box)
        self.query_combo.setMinimumWidth(150)
        self.query_combo.setEditable(True)
        query_box.layout().addWidget(self.query_combo)
        self.query_combo.activated[int].connect(self.select_query)
        self.query_combo.lineEdit().returnPressed.connect(self.run_initial_query)
        self.nyt_controls.append(self.query_combo)

        # Year box.
        year_box = gui.widgetBox(parameter_box, orientation=0)
        # Inputs for years.
        gui.label(year_box, self, "From")
        self.year_from_input = gui.lineEdit(year_box, self, "year_from")
        self.year_from_input.setPlaceholderText("YYYY/MM/DD")
        self.year_from_input.setMinimumWidth(80)
        # Calendar button from.
        open_calendar_button_from = gui.button(year_box, self, '',
                                               callback=lambda: self.open_calendar(self.year_from_input),
                                               tooltip="Pick a date using the calendar widget.")
        open_calendar_button_from.setMaximumSize(open_calendar_button_from.sizeHint())
        open_calendar_button_from.setIcon(QIcon(_i("calendar.svg")))
        open_calendar_button_from.setIconSize(QtCore.QSize(16, 16))
        open_calendar_button_from.setFocusPolicy(QtCore.Qt.NoFocus)
        open_calendar_button_from.setSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        gui.label(year_box, self, "to")
        self.year_to_input = gui.lineEdit(year_box, self, "year_to")
        self.year_to_input.setPlaceholderText("YYYY/MM/DD")
        self.year_to_input.setMinimumWidth(80)
        # Calendar button to.
        open_calendar_button_to = gui.button(year_box, self, '',
                                             callback=lambda: self.open_calendar(self.year_to_input),
                                             tooltip="Pick a date using the calendar widget.")
        open_calendar_button_to.setMaximumSize(open_calendar_button_to.sizeHint())
        open_calendar_button_to.setIcon(QIcon(_i("calendar.svg")))
        open_calendar_button_to.setIconSize(QtCore.QSize(16, 16))
        open_calendar_button_to.setFocusPolicy(QtCore.Qt.NoFocus)

        self.nyt_controls.append(self.year_from_input)
        self.nyt_controls.append(self.year_to_input)

        # Text includes box.
        self.text_includes_box = gui.widgetBox(self.controlArea,
                                               "Text includes", addSpace=True)
        self.headline_chbox = gui.checkBox(self.text_includes_box, self,
                                           "includes_headline", "Headline")
        self.lead_paragraph_chbox = gui.checkBox(self.text_includes_box, self,
                                                 "includes_lead_paragraph", "Lead Paragraph")
        self.snippet_chbox = gui.checkBox(self.text_includes_box, self,
                                          "includes_snippet", "Snippet")
        self.abstract_chbox = gui.checkBox(self.text_includes_box, self,
                                           "includes_abstract", "Abstract")
        self.keywords_chbox = gui.checkBox(self.text_includes_box, self,
                                           "includes_keywords", "Keywords")
        self.nyt_controls.append(self.headline_chbox)
        self.nyt_controls.append(self.lead_paragraph_chbox)
        self.nyt_controls.append(self.snippet_chbox)
        self.nyt_controls.append(self.abstract_chbox)
        self.nyt_controls.append(self.keywords_chbox)

        # Run query button.
        self.run_query_button = gui.button(self.controlArea, self, 'Run query',
                                           callback=self.run_initial_query,
                                           tooltip="Run the chosen NYT article query.")
        self.run_query_button.setFocusPolicy(QtCore.Qt.NoFocus)
        self.nyt_controls.append(self.run_query_button)
        h_line = QFrame()
        h_line.setFrameShape(QFrame.HLine)
        h_line.setFrameShadow(QFrame.Sunken)
        self.controlArea.layout().addWidget(h_line)

        # Query info.
        query_info_box = gui.widgetBox(self.controlArea, addSpace=True)
        self.query_info_label = gui.label(query_info_box, self, "Records: /\nRetrieved: /")

        # Retrieve other records.
        self.retrieve_other_button = gui.button(self.controlArea, self, 'Retrieve remaining records',
                                                callback=self.retrieve_remaining_records,
                                                tooltip="Retrieve the remaining records obtained in the query.")
        self.retrieve_other_button.setFocusPolicy(QtCore.Qt.NoFocus)
        self.retrieve_other_button.setEnabled(False)

        # Load the most recent queries.
        self.set_query_list()

        # Check the API key and enable controls accordingly.
        if self.recent_api_keys:
            self.api_key = self.recent_api_keys[0]
            self.check_api_key(self.api_key)

        self.enable_controls()

    def set_query_list(self):
        self.query_combo.clear()
        if not self.recent_queries:
            # Sample queries.
            self.recent_queries.append("slovenia")
            self.recent_queries.append("text mining")
            self.recent_queries.append("orange data mining")
            self.recent_queries.append("bioinformatics")
        for query in self.recent_queries:
            self.query_combo.addItem(query)

    def select_query(self, n):
        if n < len(self.recent_queries):
            name = self.recent_queries[n]
            del self.recent_queries[n]
            self.recent_queries.insert(0, name)
        else:
            self.recent_queries.insert(0, self.query_combo.currentText())

        if len(self.recent_queries) > 0:
            self.set_query_list()

    def update_validity_icon(self):
        if self.api_key_is_valid:
            self.api_key_valid_label.setPixmap(QPixmap(_i("valid.svg"))
                                               .scaled(15, 15, QtCore.Qt.KeepAspectRatio))
        else:
            self.api_key_valid_label.setPixmap(QPixmap(_i("invalid.svg"))
                                               .scaled(15, 15, QtCore.Qt.KeepAspectRatio))

    def run_initial_query(self):
        self.warning(1)
        self.error(1)
        # Only execute if the NYT object is present(safety lock).
        # Otherwise this method cannot be called anyway.
        if not self.nyt_api:
            return

        # Interrupts on faulty inputs. #
        # Query keywords.
        qkw = self.query_combo.currentText()
        if not qkw:
            self.warning(1, "Please enter a query before attempting to fetch results.")
            return

        # Text fields.
        text_includes_params = [self.includes_headline, self.includes_lead_paragraph, self.includes_snippet,
                                self.includes_abstract, self.includes_keywords]
        if True not in text_includes_params:
            self.warning(1, "You must select at least one text field.")
            return

        # Year span.
        date_from = self.validate_date(self.year_from)
        date_to = self.validate_date(self.year_to)

        # Warnings on bad inputs. #
        if date_from is not None:
            if date_from < date(1851, 1, 1):
                date_from = date(1851, 1, 1)
                self.warning(1, "There are no records before the year 1851. "
                                "Assumed 1851/01/01 as start date for this query.")
            if date_to is not None:
                if date_from > date_to:
                    self.warning(1, "The start date is greater than the end date.")

        # Set the query url.
        required_text_fields = [tp for tf, tp in zip(text_includes_params, NYT_TEXT_FIELDS) if tf]
        self.nyt_api._set_endpoint_url(qkw, date_from, date_to, required_text_fields)

        # Execute the query.
        res, cached, error = self.nyt_api._execute_query(0)

        # Display error if failure.
        if self.display_error_response(res, error):
            return

        # Construct a corpus for the output.
        corpus = _generate_corpus(res["response"]["docs"], required_text_fields)
        self.output_corpus = corpus
        self.send(Output.CORPUS, self.output_corpus)

        # Update the response info.
        self.all_hits = res["response"]["meta"]["hits"]
        self.num_retrieved = len(res["response"]["docs"])
        self.update_info_label()

        # Enable 'retrieve remaining' button.
        if self.num_retrieved < min(self.all_hits, 1000):
            self.retrieve_other_button.setText('Retrieve remaining {} records'
                                               .format(min(self.all_hits, 1000)-self.num_retrieved))
            self.retrieve_other_button.setEnabled(True)
            #self.retrieve_other_button.setFocus()
        else:
            self.retrieve_other_button.setText('All records retrieved')
            self.retrieve_other_button.setEnabled(False)

        # Add the query to history.
        if qkw not in self.recent_queries:
            self.recent_queries.insert(0, qkw)
            self.select_query(0)

    def retrieve_remaining_records(self):
        self.error(1)
        # If a query is running, stop it.
        if self.query_running:
            self.query_running = False
            return

        if not self.nyt_api:
            return

        num_steps = min(math.ceil(self.all_hits/10), 100)

        # Update buttons.
        self.retrieve_other_button.setText('Stop retrieving')
        self.open_set_api_key_dialog_button.setEnabled(False)
        self.run_query_button.setEnabled(False)

        self.query_running = True
        self.progressBarInit()
        for i in range(int(self.num_retrieved/10), num_steps):
            # Stop querying if the flag is not set.
            if not self.query_running:
                break

            res, cached, error = self.nyt_api._execute_query(i)

            # Display error if failure.
            if self.display_error_response(res, error):
                return

            metas, class_values = _parse_record_json(res["response"]["docs"], self.nyt_api.includes_fields)
            docs = []
            for doc in metas:
                docs.append(" ".join([d for d in doc if d is not None]).strip())

            # Update the corpus.
            self.output_corpus.extend_corpus(docs, metas, class_values)

            # Update the info label.
            self.num_retrieved += len(res["response"]["docs"])
            self.update_info_label()

            if not cached:  # Only wait if an actual request was made.
                sleep(1)

            # Update the progress bar.
            self.progressBarSet(100.0 * (i/num_steps))

        self.progressBarFinished()
        self.query_running = False

        self.send(Output.CORPUS, self.output_corpus)

        if self.num_retrieved == min(self.all_hits, 1000):
            self.retrieve_other_button.setText('All available records retrieved')
            self.retrieve_other_button.setEnabled(False)
        else:
            self.retrieve_other_button.setText('Retrieve remaining {} records'
                                               .format(min(self.all_hits, 1000)-self.num_retrieved))
            self.retrieve_other_button.setFocus()

        self.open_set_api_key_dialog_button.setEnabled(True)
        self.run_query_button.setEnabled(True)

    def update_info_label(self):
        info_label = "Records: {}\nRetrieved: {}".format(self.all_hits, self.num_retrieved)
        if self.all_hits > 1000:
            info_label += " (max 1000)"
        self.query_info_label.setText(info_label)

    def display_error_response(self, res, error):
        failure = (not res) or ('response' not in res) or ('docs' not in res['response'])
        if failure and error is not None:
            if isinstance(error, HTTPError):
                self.error(1, "An error occurred (HTTP {})".format(error.code))
            elif isinstance(error, URLError):
                self.error(1, "An error occurred (URL {})".format(error.reason))
        return failure

    def validate_date(self, input_string):
        if not input_string or input_string is None:
            return None  # Nothing to do here.

        format_mapping = {1: "%Y", 2: "%Y/%m", 3: "%Y/%m/%d"}
        # Check for input formats: YYYY, YYYY/MM or YYYY/MM/DD.
        try:
            f = format_mapping.get(len(input_string.split('/')), '')
            return datetime.strptime(input_string, f).date()
        except ValueError:
            self.warning(1, "Invalid date interval endpoint format (must be YYYY/MM/DD).")
            return None

    def check_api_key(self, api_key):
        nyt_api = NYT(api_key)
        key_valid_flag = nyt_api.check_api_key()

        if key_valid_flag:
            self.api_key = api_key

        self.api_key_updated(key_valid_flag)

    def api_key_updated(self, is_valid):
        self.api_key_is_valid = is_valid
        self.enable_controls()

        self.update_validity_icon()
        if is_valid:
            self.nyt_api = NYT(self.api_key)    # Set the NYT API object, if key is valid.

    def enable_controls(self):
        for control in self.nyt_controls:
            control.setEnabled(self.api_key_is_valid)
        if not self.api_key_is_valid:
            self.open_set_api_key_dialog_button.setFocus()

    def open_set_api_key_dialog(self):
        api_dlg = APIKeyDialog(self, "New York Times API key")
        api_dlg.exec_()

    def open_calendar(self, widget):
        cal_dlg = CalendarDialog(self, "to", "Date picker")
        if cal_dlg.exec_():
            widget.setText(cal_dlg.picked_date)


class APIKeyDialog(QDialog):
    def __init__(self, parent, windowTitle="New York Times API key"):
        super().__init__(parent, windowTitle=windowTitle)

        self.parent = parent

        self.setLayout(QVBoxLayout())
        self.layout().setMargin(10)
        self.mainArea = gui.widgetBox(self)
        self.layout().addWidget(self.mainArea)

        # Combo box.
        self.api_key_combo = QComboBox(self.mainArea)
        self.api_key_combo.setEditable(True)
        self.api_key_combo.activated[int].connect(self.select_api_key)
        self.mainArea.layout().addWidget(self.api_key_combo)

        # Buttons
        self.button_box = gui.widgetBox(self.mainArea, orientation="horizontal")
        gui.button(self.button_box, self, "Check", self.check_api_key)
        gui.button(self.button_box, self, "OK", self.accept_changes)
        gui.button(self.button_box, self, "Cancel", self.reject_changes)

        # Label
        self.label_box = gui.widgetBox(self, orientation="horizontal")
        self.api_key_check_label = gui.label(self.label_box, self, "")
        self.update_validity_label()

        # Load the most recent API keys.
        self.set_key_list()

    def set_key_list(self):
        self.api_key_combo.clear()
        for key in self.parent.recent_api_keys:
            self.api_key_combo.addItem(key)

    def update_validity_label(self):
        if self.parent.api_key_is_valid:
            self.api_key_check_label.setText("API key is valid.")
        else:
            self.api_key_check_label.setText("API key not validated!")

    def check_api_key(self):
        self.parent.check_api_key(self.api_key_combo.currentText())
        self.update_validity_label()

    def accept_changes(self):
        self.parent.check_api_key(self.api_key_combo.currentText())  # On OK check the API key also.

        if self.api_key_combo.currentText() not in self.parent.recent_api_keys \
                and self.parent.api_key_is_valid:
            self.parent.recent_api_keys.append(self.api_key_combo.currentText())
        QDialog.accept(self)

    def reject_changes(self):
        QDialog.reject(self)

    def select_api_key(self, n):
        if n < len(self.parent.recent_api_keys):
            key = self.parent.recent_api_keys[n]
            del self.parent.recent_api_keys[n]
            self.parent.recent_api_keys.insert(0, key)

        if len(self.parent.recent_api_keys) > 0:
            self.set_key_list()


class CalendarDialog(QDialog):

    picked_date = None
    source = None
    parent = None

    def __init__(self, parent, source, windowTitle="Date picker"):
        super().__init__(parent, windowTitle=windowTitle)

        self.source = source
        self.parent = parent

        self.setLayout(QVBoxLayout())
        self.mainArea = gui.widgetBox(self)
        self.layout().addWidget(self.mainArea)

        self.cal = QtGui.QCalendarWidget(self)
        self.cal.setGridVisible(True)
        self.cal.move(20, 20)
        self.cal.clicked[QtCore.QDate].connect(self.set_date)
        self.mainArea.layout().addWidget(self.cal)

        # Set the default date.
        self.picked_date = self.cal.selectedDate().toString('yyyy/MM/dd')

        gui.button(self.mainArea, self, "OK", lambda: QDialog.accept(self))

    def set_date(self, date):
        self.picked_date = date.toString('yyyy/MM/dd')



from PyQt4 import QtGui
import numpy as np

from Orange.widgets import gui
from Orange.widgets.settings import Setting
from Orange.widgets.widget import OWWidget
from Orange.data import Table
from orangecontrib.text.stats import false_discovery_rate, hypergeom_p_values


class OWWordEnrichment(OWWidget):
    # Basic widget info
    name = "Word Enrichment"
    description = "Word enrichment analysis for selected documents."
    icon = "icons/SetEnrichment.svg"
    priority = 60

    # Input/output
    inputs = [("Data", Table, "set_data"),
              ("Selected Data", Table, "set_data_selected")]
    want_main_area = True

    # Settings
    filter_by_p = Setting(False)
    filter_p_value = Setting(0.01)
    filter_by_fdr = Setting(True)
    filter_fdr_value = Setting(0.2)

    def __init__(self):
        super().__init__()

        # Init data
        self.data = None
        self.selected_data = None
        self.selected_data_transformed = None   # used for transforming the 'selected data' into the 'data' domain

        self.words = []
        self.p_values = []
        self.fdr_values = []

        # Info section
        fbox = gui.widgetBox(self.controlArea, "Info")
        self.info_all = gui.label(fbox, self, 'Cluster words:')
        self.info_sel = gui.label(fbox, self, 'Selected words:')
        self.info_fil = gui.label(fbox, self, 'After filtering:')

        # Filtering settings
        fbox = gui.widgetBox(self.controlArea, "Filter")
        hbox = gui.widgetBox(fbox, orientation=0)

        self.chb_p = gui.checkBox(hbox, self, "filter_by_p", "p-value",
                                  callback=self.filter_and_display,
                                  tooltip="Filter by word p-value")
        self.spin_p = gui.doubleSpin(hbox, self, 'filter_p_value',
                                     1e-4, 1, step=1e-4, labelWidth=15,
                                     callback=self.filter_and_display,
                                     callbackOnReturn=True,
                                     tooltip="Max p-value for word")
        self.spin_p.setEnabled(self.filter_by_p)

        hbox = gui.widgetBox(fbox, orientation=0)
        self.chb_fdr = gui.checkBox(hbox, self, "filter_by_fdr", "FDR",
                                    callback=self.filter_and_display,
                                    tooltip="Filter by word FDR")
        self.spin_fdr = gui.doubleSpin(hbox, self, 'filter_fdr_value',
                                       1e-4, 1, step=1e-4, labelWidth=15,
                                       callback=self.filter_and_display,
                                       callbackOnReturn=True,
                                       tooltip="Max p-value for word")
        self.spin_fdr.setEnabled(self.filter_by_fdr)
        gui.rubber(self.controlArea)

        # Word's list view
        self.cols = ['Word', 'p-value', 'FDR']
        self.sig_words = QtGui.QTreeWidget()
        self.sig_words.setColumnCount(len(self.cols))
        self.sig_words.setHeaderLabels(self.cols)
        self.sig_words.setSortingEnabled(True)
        self.sig_words.setSelectionMode(QtGui.QTreeView.ExtendedSelection)
        self.sig_words.sortByColumn(2, 0)   # 0 is ascending order
        for i in range(len(self.cols)):
            self.sig_words.resizeColumnToContents(i)
        self.mainArea.layout().addWidget(self.sig_words)

    def set_data(self, data=None):
        self.data = data

    def set_data_selected(self, data=None):
        self.selected_data = data

    def handleNewSignals(self):
        self.check_data()

    def check_data(self):
        self.warning(1)
        if isinstance(self.data, Table) and \
                isinstance(self.selected_data, Table):
            self.selected_data_transformed = Table.from_table(self.data.domain, self.selected_data)
            if self.selected_data_transformed.X.size > 0 and \
                    not np.isnan(self.selected_data_transformed.X).all():
                self.apply()
            else:
                self.clear()
                self.warning(1, 'No features overlap!')
        else:
            self.clear()

    def clear(self):
        self.sig_words.clear()
        self.info_all.setText('Cluster words:')
        self.info_sel.setText('Selected words:')
        self.info_fil.setText('After filtering:')

    def filter_enabled(self, b):
        self.chb_p.setEnabled(b)
        self.chb_fdr.setEnabled(b)
        self.spin_p.setEnabled(b)
        self.spin_fdr.setEnabled(b)

    def filter_and_display(self):
        self.spin_p.setEnabled(self.filter_by_p)
        self.spin_fdr.setEnabled(self.filter_by_fdr)
        self.sig_words.clear()
        count = 0
        if self.words:
            for word, pval, fval in zip(self.words, self.p_values, self.fdr_values):
                if (not self.filter_by_p or pval <= self.filter_p_value) and \
                        (not self.filter_by_fdr or fval <= self.filter_fdr_value):
                    it = EATreeWidgetItem(word, pval, fval, self.sig_words)
                    self.sig_words.addTopLevelItem(it)
                    count += 1

        for i in range(len(self.cols)):
            self.sig_words.resizeColumnToContents(i)

        self.info_all.setText('Cluster words: {}'.format(len(self.selected_data_transformed.domain.attributes)))
        self.info_sel.setText('Selected words: {}'.format(np.count_nonzero(np.sum(self.selected_data_transformed.X, axis=0))))
        if not self.filter_by_p and not self.filter_by_fdr:
            self.info_fil.setText('After filtering:')
            self.info_fil.setEnabled(False)
        else:
            self.info_fil.setEnabled(True)
            self.info_fil.setText('After filtering: {}'.format(count))

    def progress(self, p):
        self.progressBarSet(p)

    def apply(self):
        self.clear()
        self.progressBarInit()
        self.filter_enabled(False)

        self.words = [i.name for i in self.selected_data_transformed.domain.attributes]
        self.p_values = hypergeom_p_values(self.data.X,
                                           self.selected_data_transformed.X,
                                           callback=self.progress)
        self.fdr_values = false_discovery_rate(self.p_values)
        self.filter_and_display()
        self.filter_enabled(True)
        self.progressBarFinished()


fp = lambda score: "%0.5f" % score if score > 10e-3 else "%0.1e" % score
fpt = lambda score: "%0.9f" % score if score > 10e-3 else "%0.5e" % score


class EATreeWidgetItem(QtGui.QTreeWidgetItem):
    def __init__(self, word, p_value, f_value, parent):
        super().__init__(parent)
        self.data = [word, p_value, f_value]
        self.setText(0, word)
        self.setText(1, fp(p_value))
        self.setToolTip(1, fpt(p_value))
        self.setText(2, fp(f_value))
        self.setToolTip(2, fpt(f_value))

    def __lt__(self, other):
        col = self.treeWidget().sortColumn()
        return self.data[col] < other.data[col]



from PyQt4 import QtCore, QtGui

from Orange.widgets.widget import OWWidget
from Orange.widgets.settings import Setting
from Orange.widgets import gui
from orangecontrib.text.preprocess import Preprocessor


class Output:
    PREPROCESSOR = "Preprocessor"

class OWPreprocess(OWWidget):
    name = "Preprocess"
    description = "Choose the pre-processing options and return a Preprocessor object."
    icon = "icons/TextPreprocess.svg"
    priority = 30

    outputs = [(Output.PREPROCESSOR, Preprocessor)]
    want_main_area = False

    include_punctuation = Setting(False)
    lowercase = Setting(True)
    remove_stpwrds = Setting(True)
    transformation_opt = Setting(["(none)", "Stemmer", "Lemmatizer"])

    def __init__(self):
        super().__init__()

        self.transformation = None

        # Settings.
        settings_box = gui.widgetBox(self.controlArea, "Basic options", addSpace=True)

        gui.checkBox(settings_box, self, "include_punctuation", "Include punctuation")
        gui.checkBox(settings_box, self, "lowercase", "To lowercase")
        gui.checkBox(settings_box, self, "remove_stpwrds", "Remove stopwords")

        # Transformation.
        transformation_box = gui.widgetBox(self.controlArea, "Transformation", addSpace=True)

        self.trans_combo = QtGui.QComboBox(settings_box)
        transformation_box.layout().addWidget(self.trans_combo, QtCore.Qt.AlignRight)
        self.fill_transformation_options()  # Add available options.
        self.trans_combo.activated[int].connect(self.select_transformation)

        gui.button(self.controlArea, self, "&Apply", callback=self.apply, default=True)

        self.apply()

    def select_transformation(self, n):
        if n > 0:
            self.transformation = self.transformation_opt[n]
        else:
            self.transformation = None

    def fill_transformation_options(self):
        self.trans_combo.clear()
        for trans in self.transformation_opt:
            if trans == "(none)":
                self.trans_combo.addItem("(none)")
            else:
                self.trans_combo.addItem(trans)

    def apply(self):
        # TODO change this to custom stopwords
        if self.remove_stpwrds:
            sw = 'english'
        pp = Preprocessor(incl_punct=self.include_punctuation, trans=self.transformation,
                          lowercase=self.lowercase, stop_words=sw)
        self.send(Output.PREPROCESSOR, pp)


# coding: utf-8
from collections import defaultdict, Counter
from os import path
from math import pi as PI
import re

import numpy as np

from PyQt4 import QtCore, QtGui

from Orange.widgets import widget, gui, settings
from Orange.data import Table
from orangecontrib.text.country_codes import \
    CC_EUROPE, INV_CC_EUROPE, SET_CC_EUROPE, \
    CC_WORLD, INV_CC_WORLD, \
    CC_USA, INV_CC_USA, SET_CC_USA


CC_NAMES = re.compile('[\w\s\.\-]+')


class Map:
    WORLD = 'world_mill_en'
    EUROPE = 'europe_mill_en'
    USA = 'us_aea_en'
    all = (('World',  WORLD),
           ('Europe', EUROPE),
           ('USA',    USA))


class OWGeoMap(widget.OWWidget):
    name = "GeoMap"
    priority = 20000
    icon = "icons/GeoMap.svg"
    inputs = [("Data", Table, "on_data")]
    outputs = [('Data', Table)]

    want_main_area = False

    selected_attr = settings.Setting(0)
    selected_map = settings.Setting(0)
    regions = settings.Setting([])

    def __init__(self):
        super().__init__()
        self._create_layout()

    @QtCore.pyqtSlot(str, result=str)
    def region_selected(self, regions):
        """Called from JavaScript"""
        if not regions:
            self.regions = []
            return self.send('Data', None)
        self.regions = regions.split(',')
        attr = self.metas[self.selected_attr]
        if attr.is_discrete: return  # TODO, FIXME: make this work for discrete attrs also
        from Orange.data.filter import FilterRegex
        filter = FilterRegex(attr, r'\b{}\b'.format(r'\b|\b'.join(self.regions)), re.IGNORECASE)
        self.send('Data', self.data._filter_values(filter))

    def _create_layout(self):
        box = gui.widgetBox(self.controlArea,
                            orientation='horizontal')
        self.attr_combo = gui.comboBox(box, self, 'selected_attr',
                                       orientation='horizontal',
                                       label='Region attribute:',
                                       callback=self.on_attr_change)
        self.map_combo = gui.comboBox(box, self, 'selected_map',
                                      orientation='horizontal',
                                      label='Map type:',
                                      callback=self.on_map_change,
                                      items=Map.all)
        hexpand = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding,
                                    QtGui.QSizePolicy.Fixed)
        self.attr_combo.setSizePolicy(hexpand)
        self.map_combo.setSizePolicy(hexpand)
        html = '''
<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8">
<base href="file://{}/"/>
<style>
html, body, #map {{margin:0px;padding:0px;width:100%;height:100%;}}
</style>
<link  href="resources/jquery-jvectormap-2.0.2.css" rel="stylesheet">
</head>
<body>
<div id="map"></div>
</body>
</html>'''.format(path.abspath(path.dirname(__file__)))
        self.webview = gui.WebviewWidget(self.controlArea, self, debug=True)
        self.webview.setHtml(html)
        for script in ('jquery-2.1.4.min.js',
                       'jquery-jvectormap-2.0.2.min.js',
                       'jquery-jvectormap-world-mill-en.js',
                       'jquery-jvectormap-europe-mill-en.js',
                       'jquery-jvectormap-us-aea-en.js',
                       'geomap-script.js'):
            self.webview.evalJS(open(path.join(path.dirname(__file__), 'resources', script), encoding='utf-8').read())
        self.webview.evalJS('REGIONS = {};'.format({Map.WORLD: CC_WORLD,
                                                    Map.EUROPE: CC_EUROPE,
                                                    Map.USA: CC_USA}))

    def _repopulate_attr_combo(self, data):
        from itertools import chain
        self.metas = [a for a in chain(data.domain.metas,
                                       data.domain.attributes,
                                       data.domain.class_vars)
                      # Filter string variables
                      if (a.is_discrete and a.values and isinstance(a.values[0], str) and not a.ordered or
                          a.is_string)] if data else []
        self.attr_combo.clear()
        self.selected_attr = 0
        for i, var in enumerate(self.metas):
            self.attr_combo.addItem(gui.attributeIconDict[var], var.name)
            # Select default attribute
            if var.name.lower() == 'country':
                self.selected_attr = i
        if self.metas:
            self.attr_combo.setCurrentIndex(self.attr_combo.findText(self.metas[self.selected_attr].name))

    def on_data(self, data):
        self.data = data
        self._repopulate_attr_combo(data)
        if not data:
            self.region_selected('')
            self.webview.evalJS('DATA = {}; renderMap();')
        else:
            self.on_attr_change()

    def on_map_change(self, map_code=''):
        if map_code:
            self.map_combo.setCurrentIndex(self.map_combo.findData(map_code))
        else:
            map_code = self.map_combo.itemData(self.selected_map)

        inv_cc_map, cc_map = {Map.USA: (INV_CC_USA, CC_USA),
                              Map.WORLD: (INV_CC_WORLD, CC_WORLD),
                              Map.EUROPE: (INV_CC_EUROPE, CC_EUROPE)} [map_code]
        # Set country counts in JS
        data = defaultdict(int)
        for cc in self.cc_counts:
            key = inv_cc_map.get(cc, cc)
            if key in cc_map:
                data[key] += self.cc_counts[cc]
        self.webview.evalJS('DATA = {};'.format(dict(data)))
        # Draw the new map
        self.webview.evalJS('MAP_CODE = "{}";'.format(map_code))
        self.webview.evalJS('SELECTED_REGIONS = {};'.format(self.regions))
        self.webview.evalJS('renderMap();')

    def on_attr_change(self):
        attr = self.metas[self.selected_attr]
        if attr.is_discrete:
            return self.warning(0, 'Discrete region attributes not yet supported. Patches welcome!')
        countries = (set(map(str.strip, CC_NAMES.findall(i.lower()))) if len(i) > 3 else (i,)
                     for i in self.data.get_column_view(self.data.domain.index(attr))[0])
        def flatten(seq):
            return (i for sub in seq for i in sub)
        self.cc_counts = Counter(flatten(countries))
        # Auto-select region map
        values = set(self.cc_counts)
        if 0 == len(values - SET_CC_USA):
            map_code = Map.USA
        elif 0 == len(values - SET_CC_EUROPE):
            map_code = Map.EUROPE
        else:
            map_code = Map.WORLD
        self.on_map_change(map_code)


def main():
    from Orange.data import Table, Domain, ContinuousVariable, StringVariable

    words = np.column_stack([
        'Slovenia Slovenia SVN USA Iraq Iraq Iraq Iraq France FR'.split(),
        'Slovenia Slovenia SVN France FR Austria NL GB GB GB'.split(),
        'Alabama AL Texas TX TX TX MS Montana US-MT MT'.split(),
    ])
    metas = [
        StringVariable('World'),
        StringVariable('Europe'),
        StringVariable('USA'),
    ]
    domain = Domain([], metas=metas)
    table = Table.from_numpy(domain,
                             X=np.zeros((len(words), 0)),
                             metas=words)
    app = QtGui.QApplication([''])
    w = OWGeoMap()
    w.on_data(table)
    w.show()
    app.exec()


if __name__ == "__main__":
    main()



from Orange.data.table import Table


class Topics(Table):
    """
        Dummy wrapper for Table so signals can distinguish Topics from Data.
    """
    def __new__(cls, *args, **kwargs):
        """Bypass Table.__new__."""
        return object.__new__(Topics)



import os
import tempfile
import unittest
import datetime
from unittest.mock import patch
from contextlib import contextmanager
from orangecontrib.text.nyt import NYT, NYT_TEXT_FIELDS

from Orange.data import StringVariable


class MockUrlOpen:
    def __init__(self, cache):
        self.data = {}
        with open(cache, 'r') as f:
            for url in f:
                data = next(f)
                self.data[url.strip()] = data.strip().encode('utf-8')
                try:
                    next(f)
                except StopIteration:
                    # Last empty line is sometimes missing
                    pass

    def __call__(self, url):
        self.current_url = url
        self.response_code = 200 if url in self.data else 404

        @contextmanager
        def cm():
            yield self

        return cm()

    def readall(self):
        return self.data[self.current_url]

    def getcode(self):
        return self.response_code


CACHE = os.path.join(os.path.dirname(__file__), 'nyt-cache.txt')
mock_urllib = MockUrlOpen(CACHE)


@patch('urllib.request.urlopen', mock_urllib)
class NYTTests(unittest.TestCase):
    API_KEY = 'api_key'

    def setUp(self):
        self.tmp = tempfile.NamedTemporaryFile(delete=False)
        os.remove(self.tmp.name)
        self.nyt = NYT(self.API_KEY)
        self.nyt.cache_path = self.tmp.name

    def tearDown(self):
        if os.path.exists(self.tmp.name):
            os.remove(self.tmp.name)

    def test_nyt_key(self):
        self.assertTrue(self.nyt.check_api_key())

    def test_nyt_query_keywords(self):
        records = self.nyt.run_query('slovenia')
        self.assertEqual(len(records), 10)

    def test_nyt_query_date_range(self):
        corpus = self.nyt.run_query('slovenia', datetime.date(2013, 1, 1), datetime.date(2014, 1, 1))
        self.assertEqual(len(corpus.documents), 10)

    def test_nyt_query_max_records(self):
        records = self.nyt.run_query('slovenia', max_records=25)
        self.assertEqual(len(records), 25)

    def test_nyt_corpus_domain_generation(self):
        corpus = self.nyt.run_query('slovenia')
        meta_vars = [StringVariable.make(field) for field in NYT_TEXT_FIELDS] + \
                    [StringVariable.make('pub_date'), StringVariable.make('country')]

        self.assertEqual(len(meta_vars), len(corpus.domain.metas))
        self.assertEqual(len(corpus.Y), 10)

    def test_nyt_result_caching(self):
        # Run a query to create a cache entry first.
        self.nyt.run_query('slovenia')
        data, is_cached, error = self.nyt._execute_query(0)
        # Check if the response was cached.
        self.assertTrue(is_cached)


from nltk.stem.porter import PorterStemmer
from nltk.stem import WordNetLemmatizer
from sklearn.feature_extraction.text import CountVectorizer


class Preprocessor:
    """
        Holds pre-processing flags and other information, about stop word
        removal, lowercasing, text morphing etc.(the options are set via
        the Preprocess widget).
    """
    def __init__(self, incl_punct=False, lowercase=True, stop_words='english', trans=None, min_df=1):
        """
        :param incl_punct: Determines whether the tokenizer should include punctuation in the tokens.
        :type incl_punct: boolean
        :param lowercase: If set, transform the tokens to lower case, before returning them.
        :type lowercase: boolean
        :param stop_words: Determines whether stop words should("english"), or should not(None) be removed.
            If this is list, it should contain stopwords.
        :type stop_words: 'english' or list or None
        :param trans: An optional pre-processor object to perform the morphological
            transformation on the tokens before returning them.
        :type trans: :class: `orangecontrib.text.preprocess.Lemmatizer`
            or :class: `orangecontrib.text.preprocess.Stemmer`
        :return: :class: `orangecontrib.text.preprocess.Preprocessor`
        """
        # TODO Needs more elaborate check on list contents.
        if not (stop_words == 'english' or isinstance(stop_words, list) or stop_words is None):
            raise ValueError("The stop words parameter should be either \'english\', a list or None.")
        if isinstance(trans, Stemmatizer):  # hack, since scikit does not obey lowercase when preprocesor given
            trans.lowercase = lowercase
        self.stop_words = stop_words
        # TODO does including punctuation even work?
        self.incl_punct = incl_punct
        self.lowercase = lowercase
        self.transformation = trans
        self.min_df = min_df
        self.cv = CountVectorizer(
            lowercase=lowercase,
            stop_words=stop_words,
            preprocessor=trans,
            min_df=min_df,
        )

    def __call__(self, data):
        if isinstance(data, str):
            vec = self.cv.fit([data])
            return vec.get_feature_names()
        if isinstance(data, list):
            vec = self.cv.fit(data)
            features = vec.get_feature_names()
            docs = [[] for _ in range(len(data))]
            for (line, column), count in vec.transform(data).todok().items():
                docs[line].extend([features[column]] * count)
            return docs
        else:
            raise ValueError("Type '{}' not supported.".format(type(data)))


class Stemmatizer():
    """
        A common class for stemming and lemmatization.
    """
    def __init__(self, trans, lowercase=True, name='Stemmatizer'):
        """
            :param trans: The method that will perform transformation on the tokens.
            :param name: The name of the transformation object.
            :return: :class: `orangecontrib.text.preprocess.Stemmatizer`
        """
        self.trans = trans
        self.name = name
        self.lowercase = lowercase

    def __call__(self, data):
        """
            :param data: The input that we wish to transform.
            :type data: string, list or :class: `Orange.data.Table`
            :return: string, list or :class: `Orange.data.Table`
        """
        if isinstance(data, str):
            if self.lowercase:
                return self.trans(data).lower()
            else:
                return self.trans(data)
        elif isinstance(data, list):
            if self.lowercase:
                return [self.trans(word).lower() for word in data]
            else:
                return [self.trans(word) for word in data]
        else:
            raise ValueError("Type {} not supported.".format(type(data)))

Stemmer = Stemmatizer(PorterStemmer().stem, 'Stemmer')
Lemmatizer = Stemmatizer(WordNetLemmatizer().lemmatize, 'Lemmatizer')



import os
import math
import json
import shelve
import warnings
import datetime
import numpy as np
from datetime import date
from urllib import request, parse
from urllib.error import HTTPError, URLError
from orangecontrib.text.corpus import Corpus

from Orange.canvas.utils import environ
from Orange.data import Domain, StringVariable, DiscreteVariable

NYT_TEXT_FIELDS = ["headline", "lead_paragraph", "snippet", "abstract", "keywords"]


def _parse_record_json(records, includes_metadata):
    """
    Parses the JSON representation of the record returned by the New York Times Article API.
    :param records: A list of the query's results.
    :type records: list
    :param includes_metadata: The flags that determine which fields to include.
    :type includes_metadata: list
    :return: A list of the corresponding metadata and class values for the instances.
    """
    class_values = []
    metadata = np.empty((0, len(includes_metadata)+2), dtype=object)    # columns: Text fields + (pub_date + country)
    for doc in records:
        metas_row = []
        for field in includes_metadata:
            if field in doc:
                field_value = ""
                if isinstance(doc[field], dict):
                    field_value = " ".join([val for val in doc[field].values() if val])
                elif isinstance(doc[field], list):
                    field_value = " ".join([kw["value"] for kw in doc[field] if kw])
                else:
                    if doc[field]:
                        field_value = doc[field]
                metas_row.append(field_value)
        # Add the pub_date.
        metas_row.append(doc.get("pub_date", ""))
        # Add the glocation.
        metas_row.append(",".join([kw["value"] for kw in doc["keywords"] if kw["name"] == "glocations"]))

        # Add the section_name.
        class_values.append(doc.get("section_name", None))

        metas_row = [None if v is "" else v for v in metas_row]
        metadata = np.vstack((metadata, np.array(metas_row)))

    return metadata, class_values


def _date_to_str(input_date):
    """
    Returns a string representation of the input date, according to the ISO 8601 format.
    :param input_date:
    :type input_date: datetime
    :return: str
    """
    iso = input_date.isoformat()
    date_part = iso.strip().split("T")[0].split("-")
    return "%s%s%s" % (date_part[0], date_part[1], date_part[2])


def _generate_corpus(records, required_text_fields):
    """
    Generates a corpus from the input NYT records.
    :param records: The input NYT records.
    :type records: list
    :param required_text_fields: A list of the available NYT text fields.
    :type required_text_fields: list
    :return: :class: `orangecontrib.text.corpus.Corpus`
    """
    metas, class_values = _parse_record_json(records, required_text_fields)
    documents = []
    for doc in metas:
        documents.append(" ".join([d for d in doc if d is not None]).strip())

    # Create domain.
    meta_vars = [StringVariable.make(field) for field in required_text_fields]
    meta_vars += [StringVariable.make("pub_date"), StringVariable.make("country")]
    class_vars = [DiscreteVariable("section_name", values=list(set(class_values)))]
    domain = Domain([], class_vars=class_vars, metas=meta_vars)

    Y = np.array([class_vars[0].to_val(cv) for cv in class_values])[:, None]

    return Corpus(documents, None, Y, metas, domain)


class NYT:
    """
    An Orange text mining extension class for fetching records from the NYT API.
    """
    _base_url = 'http://api.nytimes.com/svc/search/v2/articlesearch.json'

    def __init__(self, api_key):
        # For accessing the API.
        self._api_key = api_key.strip()
        # API endpoint.
        self._query_url = None
        # For caching purposes.
        self.query_key = None
        # Record fields to include.
        self.includes_fields = None

        self.cache_path = None
        cache_folder = os.path.join(environ.buffer_dir, "nytcache")
        try:
            if not os.path.exists(cache_folder):
                os.makedirs(cache_folder)
            self.cache_path = os.path.join(cache_folder, "query_cache")
        except:
            warnings.warn('Could not assemble NYT query cache path', RuntimeWarning)

    def check_api_key(self):
        """
        Checks whether the api key provided to this class instance, is valid.
        :return: True or False depending on the validation outcome.
        """
        query_url = self._encode_base_url("test")
        try:
            with request.urlopen("{0}?{1}&page=0".format(self._base_url, query_url)) as connection:
                if connection.getcode() == 200:
                    return True
        except HTTPError:
            return False

    def run_query(self, query, date_from=None, date_to=None, max_records=10):
        """
        Executes the NYT query specified by the input parameters and returns a
        list of records.
        :param query: The query keywords in a string, but separated with whitespaces.
        :type query: str
        :param date_from: Signifies to return articles from this date forth only.
        :type date_from: date
        :param date_to: Signifies to return articles up to this date only.
        :type date_to: date
        :param max_records: Specifies an upper limit to the number of retrieved records.
            Max 1000.
        :type max_records: int
        :return: list
        """
        # Check API key validity first.
        if not self.check_api_key():
            warnings.warn("Cannot execute query. The specified API key is not valid.", RuntimeWarning)

        # Collect the user inputs and assemble the API endpoint.
        self._set_endpoint_url(query, date_from, date_to, NYT_TEXT_FIELDS)

        if max_records > 1000:
            warnings.warn("Cannot retrieve more than 1000 records for a particular query.", RuntimeWarning)

        num_steps = min(math.ceil(max_records/10), 100)
        records = []
        for i in range(0, num_steps):
            data, cached, err = self._execute_query(i)
            failure = (data is None) or ('response' not in data) or ('docs' not in data['response'])
            if failure:
                warnings.warn("Warning: could not retrieve page {} of specified query.".format(i))
                continue
            records.extend(data["response"]["docs"])
            # Trim the records list.
        if len(records) > max_records:
            records = records[:(max_records-len(records))]

        return _generate_corpus(records, NYT_TEXT_FIELDS)

    def _set_endpoint_url(self, query, date_from=None, date_to=None, text_includes=None):
        """
        Builds a NYT article API query url with the input parameters.
        For more information on the inputs, refer to the docs for 'run_query' method.
        :return: None
        """
        # Query keywords, base url and API key.
        query_url = self._encode_base_url(query)
        query_key = []  # This query's key, to store with shelve.

        # Check from date.
        if date_from is None:   # Is none provided?
            query_key.append(_date_to_str(datetime.date(1851, 1, 1)))   # For caching.
        elif not isinstance(date_from, date):   # Is it in an unsupported format?
            warnings.warn('Type {} is not supported.'.format(type(date_from)), RuntimeWarning)
        else:   # Update the url and cache key.
            query_key.append(_date_to_str(date_from))
            query_url += "&begin_date=" + _date_to_str(date_from)

        # Check to date.
        if date_to is None:   # Is none provided?
            query_key.append(_date_to_str(datetime.datetime.now().date()))   # For caching.
        elif not isinstance(date_to, date):   # Is it in an unsupported format?
            warnings.warn('Type {} is not supported.'.format(type(date_to)), RuntimeWarning)
        else:   # Update the url and cache key.
            query_key.append(_date_to_str(date_to))
            query_url += "&end_date=" + _date_to_str(date_to)

        # Text fields.
        if text_includes:
            self.includes_fields = text_includes
            fl = ",".join(text_includes)
            query_url += "&fl=" + fl
        # Add pub_date.
        query_url += ",pub_date"
        # Add section_name.
        query_url += ",section_name"
        # Add keywords in every case, since we need them for geolocating.
        if "keywords" not in text_includes:
            query_url += ",keywords"

        self._query_url = "{0}?{1}".format(self._base_url, query_url)

        # Queries differ in query words, included text fields and date range.
        query_key.extend(query.split(" "))
        query_key.extend(text_includes)

        self.query_key = "_".join(query_key)

    def _execute_query(self, page):
        """
        Execute a query and get the data from the New York Times Article API.
        Will not execute, if the class object's query_url has not been set.
        :param page: Determine what page of the query to return.
        :type page: int
        :return: A JSON representation of the query's results, a boolean flag,
            that serves as feedback, whether the request was cached or not and
            the error if one occurred during execution.
        """
        if not self._query_url:
            warnings.warn('Could not find any specified queries.', RuntimeWarning)

        # Method return values.
        response_data = ""
        is_cached = False

        current_query = self._query_url+"&page={}".format(page)
        current_query_key = self.query_key + "_" + str(page)

        with shelve.open(self.cache_path) as query_cache:
            if current_query_key in query_cache.keys():
                response = query_cache[current_query_key]
                response_data = json.loads(response)
                is_cached = True
                error = None
            else:
                try:
                    with request.urlopen(current_query) as connection:
                        response = connection.readall().decode("utf-8")
                    query_cache[current_query_key] = response

                    response_data = json.loads(response)
                    is_cached = False
                    error = None
                except (HTTPError, URLError) as err:
                    error = err

            query_cache.close()    # Release resources.
        return response_data, is_cached, error

    def _encode_base_url(self, query):
        """
        Builds the foundation url string for this query.
        :param query: The keywords for this query.
        :type query: str
        :return: The foundation url for this query.
        """
        return parse.urlencode([("q", query), ("fq", "The New York Times"), ("api-key", self._api_key)])



from PyQt4 import QtGui

from Orange.widgets.widget import OWWidget
from Orange.widgets.settings import Setting
from Orange.widgets import gui
from Orange.data import Table
from Orange.widgets.data.contexthandlers import DomainContextHandler
from orangecontrib.text.corpus import Corpus
from orangecontrib.text.preprocess import Preprocessor
from orangecontrib.text.lda import LDA
from orangecontrib.text.topics import Topics


class Output:
    DATA = "Data"
    TOPICS = "Topics"


class OWLDA(OWWidget):
    # Basic widget info
    name = "Topic Discovery"
    description = "Latent Dirichlet Allocation topic model."
    icon = "icons/LDA.svg"
    priority = 50

    settingsHandler = DomainContextHandler()

    # Input/output
    inputs = [("Corpus", Corpus, "set_data"),
              ("Preprocessor", Preprocessor, "set_preprocessor")]
    outputs = [(Output.DATA, Table),
               (Output.TOPICS, Topics)]
    want_main_area = True

    # Settings
    num_topics = Setting(5)

    def __init__(self):
        super().__init__()

        self.lda = None
        self.corpus = None
        self.preprocessor = Preprocessor()

        # Info.
        info_box = gui.widgetBox(self.controlArea, "Info")
        self.info_label = gui.label(info_box, self, '')

        # Settings.
        topic_box = gui.widgetBox(self.controlArea, "Settings")
        hbox = gui.widgetBox(topic_box, orientation=0)
        self.topics_label = gui.label(hbox, self, 'Number of topics: ')
        self.topics_label.setMaximumSize(self.topics_label.sizeHint())
        self.topics_input = gui.spin(hbox, self, "num_topics",
                                     minv=1, maxv=2 ** 31 - 1,
                                     callback=self.num_topics_changed)

        # Commit button
        self.commit = gui.button(self.controlArea, self, "&Apply",
                                 callback=self.apply, default=True)
        self.commit.setEnabled(False)
        gui.rubber(self.controlArea)

        # Topics description
        self.cols = ['Topic', 'Topic keywords']
        self.topic_desc = QtGui.QTreeWidget()
        self.topic_desc.setColumnCount(len(self.cols))
        self.topic_desc.setHeaderLabels(self.cols)
        #self.topic_desc.setSelectionMode(QtGui.QTreeView.ExtendedSelection)
        self.topic_desc.itemSelectionChanged.connect(self.selected_topic_changed)
        for i in range(len(self.cols)):
            self.topic_desc.resizeColumnToContents(i)
        self.mainArea.layout().addWidget(self.topic_desc)

        self.refresh_gui()

    def set_preprocessor(self, data):
        if data is None:
            self.preprocessor = Preprocessor()
        else:
            self.preprocessor = data
        self.apply()

    def set_data(self, data=None):
        self.corpus = data
        self.apply()

    def refresh_gui(self):
        got_corpus = self.corpus is not None
        ndoc = len(self.corpus) if got_corpus else "(None)"
        self.info_label.setText("Input text entries: {}".format(ndoc))

    def update_topics(self):
        self.topic_desc.clear()
        for i in range(self.lda.num_topics):
            words = self.lda.get_top_words_by_id(i)
            it = LDATreeWidgetItem(i, words, self.topic_desc)
            self.topic_desc.addTopLevelItem(it)
        for i in range(2):
            self.topic_desc.resizeColumnToContents(i)

    def num_topics_changed(self):
        if self.corpus is None or \
                (self.lda is not None and self.lda.num_topics == self.num_topics):
            self.commit.setEnabled(False)
        else:
            self.commit.setEnabled(True)

    def selected_topic_changed(self):
        selected = self.topic_desc.selectedItems()
        if selected:
            self.send_topic_by_id(selected[0].topic_id)

    def send_topic_by_id(self, topic_id):
        self.topic_desc.setCurrentItem(self.topic_desc.topLevelItem(topic_id))
        self.send(Output.TOPICS, self.lda.get_topics_table_by_id(topic_id))

    def enabled(self, bool):
        self.topics_input.setEnabled(bool)

    def progress(self, p):
        self.progressBarSet(p)

    def apply(self):
        self.topic_desc.clear()
        self.refresh_gui()
        if self.corpus:
            self.commit.setEnabled(False)
            self.enabled(False)

            preprocessed = self.preprocessor(self.corpus.documents)

            self.progressBarInit()
            self.lda = LDA(preprocessed, num_topics=self.num_topics, callback=self.progress)
            table = self.lda.insert_topics_into_corpus(self.corpus)
            self.update_topics()
            self.progressBarFinished()

            self.send(Output.DATA, table)
            self.send_topic_by_id(0)

            self.enabled(True)
        else:
            self.send(Output.DATA, None)
            self.send(Output.TOPICS, None)


class LDATreeWidgetItem(QtGui.QTreeWidgetItem):
    def __init__(self, topic_id, words, parent):
        super().__init__(parent)
        self.topic_id = topic_id
        self.words = words

        self.setText(0, '{:d}'.format(topic_id+1))
        self.setText(1, ', '.join(words))


import os
import unittest
from orangecontrib.text.preprocess import Preprocessor, Stemmer, Lemmatizer


class PreprocessTests(unittest.TestCase):
    sentence = "Human machine interface for lab abc computer applications"
    corpus = ["Human machine interface for lab abc computer applications",
              "A survey of user opinion of computer system response time",
              "The EPS user interface management system",
              "System and human system engineering testing of EPS",
              "Relation of user perceived response time to error measurement",
              "The generation of random binary unordered trees",
              "The intersection graph of paths in trees",
              "Graph minors IV Widths of trees and well quasi ordering",
              "Graph minors A survey",
              ]

    def test_preprocess_one_sentence_lowercase_stopwords(self):
        p = Preprocessor(lowercase=True, stop_words=None)
        corpus = p(self.sentence)
        self.assertEqual(sorted(corpus),
                         sorted(['abc', 'applications', 'computer', 'for', 'human', 'interface', 'lab', 'machine']))

        p = Preprocessor(lowercase=True, stop_words='english')
        corpus = p(self.sentence)
        self.assertEqual(sorted(corpus),
                         sorted(['abc', 'applications', 'computer', 'human', 'interface', 'lab', 'machine']))

        p = Preprocessor(lowercase=True, stop_words=['abc', 'applications', 'computer', 'for'])
        corpus = p(self.sentence)
        self.assertEqual(sorted(corpus),
                         sorted(['human', 'interface', 'lab', 'machine']))

        p = Preprocessor(lowercase=False, stop_words=None)
        corpus = p(self.sentence)
        self.assertEqual(sorted(corpus),
                         sorted(['Human', 'abc', 'applications', 'computer', 'for', 'interface', 'lab', 'machine']))

        p = Preprocessor(lowercase=False, stop_words='english')
        corpus = p(self.sentence)
        self.assertEqual(sorted(corpus),
                         sorted(['abc', 'applications', 'computer', 'Human', 'interface', 'lab', 'machine']))

        p = Preprocessor(lowercase=False, stop_words=['abc', 'applications', 'computer', 'for'])
        corpus = p(self.sentence)
        self.assertEqual(sorted(corpus),
                         sorted(['Human', 'interface', 'lab', 'machine']))

    def test_preprocess_corpus_min_df(self):
        p = Preprocessor(lowercase=True, stop_words=['for', 'a', 'of', 'the', 'and', 'to', 'in'], min_df=2)
        corpus = p(self.corpus)
        correct = [['human', 'interface', 'computer'],
                   ['survey', 'user', 'computer', 'system', 'response', 'time'],
                   ['eps', 'user', 'interface', 'system'],
                   ['system', 'human', 'system', 'eps'],
                   ['user', 'response', 'time'],
                   ['trees'],
                   ['graph', 'trees'],
                   ['graph', 'minors', 'trees'],
                   ['graph', 'minors', 'survey']]
        self.assertEqual(len(corpus), len(correct))
        for i, j in zip(corpus, correct):
            self.assertEqual(sorted(i), sorted(j))

    def test_porter_stemmer(self):
        words = ['caresses', 'flies', 'dies', 'mules', 'denied',
                 'died', 'agreed', 'owned', 'humbled', 'sized',
                 'meeting', 'stating', 'siezing', 'itemization',
                 'sensational', 'traditional', 'reference', 'colonizer',
                 'plotted']
        stems = ['caress', 'fli', 'die', 'mule', 'deni', 'die', 'agre', 'own',
                 'humbl', 'size', 'meet', 'state', 'siez', 'item', 'sensat',
                 'tradit', 'refer', 'colon', 'plot']

        for w, s in zip(words, stems):
            self.assertEqual(Stemmer(w), s)

    # TODO: uncomment this test when the preprocessing  will be implemented
    #def test_porter_sentence(self):
    #    corpus = ['Caresses flies dies mules denied died agreed owned humbled sized.']
    #    stemmed = ['caress', 'fli', 'die', 'mule', 'deni', 'die', 'agre', 'own', 'humbl']

    #    p = Preprocessor(lowercase=True, stop_words=None, trans=Stemmer)
    #    corpus = p(corpus)
    #    self.assertEqual(sorted(corpus), sorted(stemmed))


    def test_wordnet_lematizer(self):
        words = ['dogs', 'churches', 'aardwolves', 'abaci', 'hardrock']
        lemas = ['dog', 'church', 'aardwolf', 'abacus', 'hardrock']

        for w, s in zip(words, lemas):
            self.assertEqual(Lemmatizer(w), s)




#!/usr/bin/env python

import os
import sys
from setuptools import setup, find_packages

NAME = 'Orange3-Text'

VERSION = '0.1.6'

DESCRIPTION = 'Orange3 TextMining add-on.'
README_FILE = os.path.join(os.path.dirname(__file__), 'README.pypi')
LONG_DESCRIPTION = open(README_FILE).read()
AUTHOR = 'Bioinformatics Laboratory, FRI UL'
AUTHOR_EMAIL = 'contact@orange.biolab.si'
URL = "https://github.com/biolab/orange3-text"
DOWNLOAD_URL = "https://github.com/biolab/orange3-text/tarball/{}".format(VERSION)

ENTRY_POINTS = {
    'orange3.addon': (
        'text = orangecontrib.text',
    ),
    # Entry point used to specify packages containing tutorials accessible
    # from welcome screen. Tutorials are saved Orange Workflows (.ows files).
    'orange.widgets.tutorials': (
        # Syntax: any_text = path.to.package.containing.tutorials
        'exampletutorials = orangecontrib.text.tutorials',
    ),

    # Entry point used to specify packages containing widgets.
    'orange.widgets': (
        # Syntax: category name = path.to.package.containing.widgets
        # Widget category specification can be seen in
        #    orangecontrib/text/widgets/__init__.py
        'Text Mining = orangecontrib.text.widgets',
    ),
}

KEYWORDS = [
    # [PyPi](https://pypi.python.org) packages with keyword "orange3 add-on"
    # can be installed using the Orange Add-on Manager
    'orange3-text',
    'data mining',
    'orange3 add-on',
]

INSTALL_REQUIRES = (
    'nltk',
    'scikit-learn',
    'numpy',
    'gensim',
)

PACKAGE_DATA = {
    'orangecontrib.text': ['tutorials/*.ows'],
    'orangecontrib.text.widgets': ['icons/*'],
    'orangecontrib.text.tests': ['nyt-cache.txt'],
}

if 'test' in sys.argv:
    extra_setuptools_args = dict(
        test_suite='orangecontrib.text.tests',
    )
else:
    extra_setuptools_args = dict()

if __name__ == '__main__':
    setup(
        name=NAME,
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        version=VERSION,
        author=AUTHOR,
        author_email=AUTHOR_EMAIL,
        url=URL,
        download_url=DOWNLOAD_URL,
        packages=find_packages(),
        package_data=PACKAGE_DATA,
        install_requires=
            INSTALL_REQUIRES + ('smart_open>=1.3.0',),
        dependency_links=(
            'git+https://github.com/piskvorky/smart_open.git@1fe93a2#egg=smart_open-1.3.0',
        ),
        entry_points=ENTRY_POINTS,
        keywords=KEYWORDS,
        namespace_packages=['orangecontrib'],
        include_package_data=True,
        zip_safe=False,
        **extra_setuptools_args
    )



# coding: utf-8
from collections import OrderedDict
from os import path
from math import pi as PI

import numpy as np

from PyQt4 import QtCore, QtGui

from Orange.widgets import widget, gui, settings
from orangecontrib.text.topics import Topics
from orangecontrib.text.corpus import Corpus
from orangecontrib.text.preprocess import Preprocessor


class SelectedWords(set):
    def __init__(self, widget):
        self.widget = widget

    def _update_webview(self):
        self.widget.webview.evalJS('SELECTED_WORDS = {};'.format(list(self)))

    def _update_filter(self):
        filter = set()
        if self.widget.bow is not None:
            for word in self:
                index = self.widget.PREPROCESS.cv.vocabulary_.get(word)
                if index is None: continue
                filter |= set(self.widget.bow[:, index].nonzero()[0])
        if filter:
            self.widget.send(Output.CORPUS, self.widget.corpus[sorted(filter), :])
        else:
            self.widget.send(Output.CORPUS, None)

    def add(self, word):
        if word not in self:
            super().add(word)
            self._update_webview()
            self._update_filter()

    def remove(self, word):
        if word in self:
            super().remove(word)
            self._update_webview()
            self._update_filter()

    def clear(self):
        super().clear()
        self._update_webview()
        self._update_filter()
        self.widget.table.clearSelection()


class Output:
    CORPUS = 'Corpus'


class OWWordCloud(widget.OWWidget):
    name = "Word Cloud"
    priority = 10000
    icon = "icons/WordCloud.svg"
    inputs = [
        ('Topics', Topics, 'on_topics_change'),
        (Output.CORPUS, Corpus, 'on_corpus_change'),
    ]
    outputs = [('Corpus', Corpus)]

    selected_words = settings.ContextSetting(SelectedWords('whatevar (?)'))
    words_color = settings.Setting(True)
    words_tilt = settings.Setting(0)

    def __init__(self):
        super().__init__()
        self.n_topic_words = 0
        self.documents_info_str = ''
        self.selected_words = SelectedWords(self)
        self.webview = None
        self.bow = None  # bag-of-words, obviously
        self.topics = None
        self.corpus = None
        self.PREPROCESS = Preprocessor()
        self._create_layout()

    @QtCore.pyqtSlot(str, result=str)
    def word_clicked(self, word):
        """Called from JavaScript"""
        if not word:
            self.selected_words.clear()
            return ''
        if word not in self.selected_words:
            self.selected_words.add(word)
            self.table.selectRowsWhere(1, word)
            return 'selected'
        else:
            self.selected_words.remove(word)
            self.table.deselectRowsWhere(1, word)
            return ''

    def _new_webview(self):
        HTML = '''
<!doctype html>
<html>
<head>
<meta charset="utf-8">
<style>
html, body {margin:0px;padding:0px;width:100%;height:100%;}
span:hover {color:OrangeRed !important}
span.selected {color:red !important}
</style>
</head>
<body id="canvas"></body>
</html>'''
        if self.webview:
            self.mainArea.layout().removeWidget(self.webview)
        self.webview = gui.WebviewWidget(self.mainArea, self, HTML, debug=True)
        for script in ('wordcloud2.js',
                       'wordcloud-script.js'):
            self.webview.evalJS(open(path.join(path.dirname(__file__), 'resources', script), encoding='utf-8').read())

    def _create_layout(self):
        self._new_webview()
        box = gui.widgetBox(self.controlArea, 'Info')
        self.topic_info = gui.label(box, self, '%(n_topic_words)d words in a topic')
        gui.label(box, self, '%(documents_info_str)s')

        box = gui.widgetBox(self.controlArea, 'Cloud preferences')
        gui.checkBox(box, self, 'words_color', 'Color words', callback=self.on_cloud_pref_change)
        TILT_VALUES = ('no', 'slight', 'more', 'full')
        gui.valueSlider(box, self, 'words_tilt', label='Words tilt:',
                        values=list(range(len(TILT_VALUES))),
                        callback=self.on_cloud_pref_change,
                        labelFormat=lambda x: TILT_VALUES[x])
        gui.button(box, None, 'Regenerate word cloud', callback=self.on_cloud_pref_change)

        box = gui.widgetBox(self.controlArea, 'Words && weights')
        self.table = gui.TableWidget(box,
                                     col_labels=['Weight', 'Word'],
                                     multi_selection=True,
                                     select_rows=True)

        def _selection_changed(selected, deselected):
            for index in deselected.indexes():
                data = self.table.rowData(index.row())
                self.selected_words.remove(data)
            for index in selected.indexes():
                data = self.table.rowData(index.row())
                self.selected_words.add(data)
            self.cloud_reselect()

        self.table.selectionChanged = _selection_changed

    def cloud_reselect(self):
        self.webview.evalJS('selectWords();')

    def on_cloud_pref_change(self):
        self._new_webview()
        self.webview.evalJS('OPTIONS["color"] = "{}"'.format(
            'random-dark' if self.words_color else 'black'))
        tilt_ratio, tilt_amount = {
            0: (0, 0),
            1: (.5,  PI/12),
            2: (.75, PI/5),
            3: (.9,  PI/2),
        }[self.words_tilt]
        self.webview.evalJS('OPTIONS["minRotation"] = {}; \
                             OPTIONS["maxRotation"] = {};'.format(-tilt_amount, tilt_amount))
        self.webview.evalJS('OPTIONS["rotateRatio"] = {};'.format(tilt_ratio))
        # Trigger cloud redrawing by constructing new webview, because everything else fail Macintosh
        self.webview.evalJS('OPTIONS["list"] = {};'.format(self.wordlist))
        self.webview.evalJS('redrawWordCloud();')


    def _repopulate_wordcloud(self, words, weights):
        N_BEST = 200
        words, weights = words[:N_BEST], weights[:N_BEST]
        # Repopulate table
        self.table.clear()
        for word, weight in zip(words, weights):
            self.table.addRow((weight, word), data=word)
        self.table.sortByColumn(0, QtCore.Qt.DescendingOrder)
        # Reset wordcloud
        mean = np.mean(weights)
        MIN_SIZE, MEAN_SIZE, MAX_SIZE = 8, 18, 40

        def _size(w):
            return np.clip(w/mean*MEAN_SIZE, MIN_SIZE, MAX_SIZE)

        self.wordlist = [[word, _size(weight)] for word, weight in zip(words, weights)]
        self.webview.evalJS('OPTIONS["list"] = {};'.format(self.wordlist))
        self.on_cloud_pref_change()

    def on_topics_change(self, data):
        self.topics = data
        self.topic_info.setVisible(bool(data))
        if not data and self.corpus:  # Topics aren't, but raw corpus is avaiable
            return self._count_words_in_corpus()
        metas = data.domain.metas if data else []
        try:
            col = next(i for i,var in enumerate(metas) if var.is_string)
        except StopIteration:
            words = np.zeros((0, 1))
        else:
            words = data.metas[:, col]
            self.n_topic_words = data.metas.shape[0]
        if not data:
            weights = np.ones(len(words))
        if data and data.W.any():
            weights = data.W[:]
        elif data and 'weights' in data.domain:
            weights = data.get_column_view(data.domain['weights'])[0]
        else:
            weights = np.ones(len(words))

        self._repopulate_wordcloud(words, weights)

    def _get_text_column(self):
        col = None
        try:
            # Take the first string meta variable
            col = next(i for i,var in enumerate(self.corpus.domain.metas) if var.is_string)
            # But prefer any that has name 'text'
            col = next(i for i,var in enumerate(self.corpus.domain.metas)
                       if var.is_string and var.name == 'text')
        except (StopIteration, AttributeError): pass
        return col

    def _bag_of_words_from_corpus(self):
        self.bow = None
        col = self._get_text_column()
        if col is None: return 0, 0
        texts = self.corpus.metas[:, col]
        self.bow = self.PREPROCESS.cv.fit_transform(texts).tocsc()
        return self.bow.shape

    def _count_words_in_corpus(self):
        col = self._get_text_column()
        if col is None: return
        words = self.PREPROCESS.cv.get_feature_names()
        freqs = np.array(self.bow.sum(axis=0))[0]
        self._repopulate_wordcloud(words, freqs)

    def on_corpus_change(self, data):
        self.corpus = data
        n_docs, n_words = self._bag_of_words_from_corpus()
        self.documents_info_str = ('{} documents with {} words'.format(n_docs, n_words)
                                   if n_docs else '(no documents on input)')
        if not self.topics:
            self._count_words_in_corpus()


def main():
    from Orange.data import Table, Domain, ContinuousVariable, StringVariable

    words = 'hey~mr. tallyman tally~me banana daylight come and me wanna go home'
    words = np.array([w.replace('~', ' ') for w in words.split()], dtype=object, ndmin=2).T
    weights = np.random.random((len(words), 1))

    data = np.zeros((len(words), 0))
    metas = []
    for i, w in enumerate(weights.T):
        data = np.column_stack((data, words, w))
        metas = metas + [StringVariable('Topic' + str(i)),
                         ContinuousVariable('weights')]
    domain = Domain([], metas=metas)
    table = Table.from_numpy(domain,
                             X=np.zeros((len(words), 0)),
                             metas=data)
    app = QtGui.QApplication([''])
    w = OWWordCloud()
    w.on_topics_change(table)
    domain = Domain([], metas=[StringVariable('text')])
    data = Corpus.from_numpy(domain, X=np.zeros((1, 0)), metas=np.array([[' '.join(words.flat)]]))
    w.on_corpus_change(data)
    w.show()
    app.exec()


if __name__ == "__main__":
    main()



import os

import numpy as np
from Orange.data import StringVariable, Table, DiscreteVariable, Domain


def get_sample_corpora_dir():
    path = os.path.dirname(__file__)
    dir = os.path.join(path, 'datasets')
    return os.path.realpath(dir)


class Corpus(Table):
    """
        Internal class for storing a corpus of orangecontrib.text.corpus.Corpus.
    """

    def __new__(cls, *args, **kwargs):
        """Bypass Table.__new__."""
        return object.__new__(Corpus)

    def __init__(self, documents, X, Y, metas, domain):
        self.documents = documents
        if X is not None:
            self.X = X
        else:
            self.X = np.zeros((len(documents), 0))
        if Y is not None:
            self.Y = Y
        else:
            self.Y = np.zeros((len(documents), 0))
        self.metas = metas
        self.W = np.zeros((len(documents), 0))
        self.domain = domain
        Table._init_ids(self)

    @classmethod
    def from_table(cls, domain, source, row_indices=...):
        c = super().from_table(domain, source, row_indices)
        c.documents = source.documents
        c._Y = c._Y.astype(np.float64)
        return c

    @classmethod
    def from_file(cls, filename):
        table = Table.from_file(filename)
        include_ids = []
        first_id = None
        for i, attr in enumerate(table.domain.metas):
            if isinstance(attr, StringVariable):
                if first_id is None:
                    first_id = i
                if attr.attributes.get('include', 'False') == 'True':
                    include_ids.append(i)
        if len(include_ids) == 0:
            include_ids.append(first_id)

        documents = []
        for line in range(table.metas.shape[0]):
            documents.append(' '.join(table.metas[line, include_ids]))

        corp = cls(documents, table.X, table.Y, table.metas, table.domain)
        corp.used_features = [f for i, f in enumerate(table.domain.metas) if i in include_ids]
        return corp

    def regenerate_documents(self, selected_features):
        documents = []
        indices = [self.domain.metas.index(f) for f in selected_features]
        for line in range(self.metas.shape[0]):
            documents.append(' '.join(self.metas[line, indices]))
        self.documents = documents

    def extend_corpus(self, documents, metadata, class_values):
        # TODO check if Domains match!
        self.metas = np.vstack((self.metas, metadata))
        self.documents += documents

        for val in set(class_values):
            if val not in self.domain.class_var.values:
                self.domain.class_var.add_value(val)
        new_Y = np.array([self.domain.class_var.to_val(cv) for cv in class_values])[:, None]
        self._Y = np.vstack((self._Y, new_Y))

        self.X = self.W = np.zeros((len(self.documents), 0))
        Table._init_ids(self)

    def __len__(self):
        return len(self.documents)



import os
from PyQt4 import QtGui

from Orange.widgets.settings import Setting
from Orange.widgets.widget import OWWidget
from Orange.widgets import gui
from Orange.widgets.data.owselectcolumns import VariablesListItemModel, VariablesListItemView
from orangecontrib.text.corpus import Corpus, get_sample_corpora_dir


class Output:
    CORPUS = "Corpus"

class OWLoadCorpus(OWWidget):
    name = "Corpus"
    description = "Load a corpus of text documents, (optionally) tagged with categories."
    icon = "icons/TextFile.svg"
    priority = 10

    outputs = [(Output.CORPUS, Corpus)]
    want_main_area = False

    dlgFormats = "Only tab files (*.tab)"

    recent_files = Setting(["(none)"])

    def __init__(self):
        super().__init__()

        self.corpus = None

        # Refresh recent files
        self.recent_files = [fn for fn in self.recent_files
                             if os.path.exists(fn)]

        # Browse file box
        fbox = gui.widgetBox(self.controlArea, "Corpus file", orientation=0)

        # Drop-down for recent files
        self.file_combo = QtGui.QComboBox(fbox)
        self.file_combo.setMinimumWidth(300)
        fbox.layout().addWidget(self.file_combo)
        self.file_combo.activated[int].connect(self.select_file)

        # Browse button
        browse = gui.button(fbox, self, 'Browse', callback=self.browse_file)
        browse.setIcon(self.style().standardIcon(QtGui.QStyle.SP_DirOpenIcon))
        browse.setSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)

        # Reload button
        reload = gui.button(fbox, self, "Reload", callback=self.reload, default=True)
        reload.setIcon(self.style().standardIcon(QtGui.QStyle.SP_BrowserReload))
        reload.setSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)

        # Corpus info
        ibox = gui.widgetBox(self.controlArea, "Corpus info", addSpace=True)
        corp_info = "Corpus of 0 documents."
        self.info_label = gui.label(ibox, self, corp_info)

        # Used Text Features
        fbox = gui.widgetBox(self.controlArea, orientation=0)
        ubox = gui.widgetBox(fbox, "Used text features", addSpace=True)
        self.used_attrs = VariablesListItemModel()
        self.used_attrs_view = VariablesListItemView()
        self.used_attrs_view.setModel(self.used_attrs)
        ubox.layout().addWidget(self.used_attrs_view)

        aa = self.used_attrs
        aa.dataChanged.connect(self.update_feature_selection)
        aa.rowsInserted.connect(self.update_feature_selection)
        aa.rowsRemoved.connect(self.update_feature_selection)

        # Ignored Text Features
        ibox = gui.widgetBox(fbox, "Ignored text features", addSpace=True)
        self.unused_attrs = VariablesListItemModel()
        self.unused_attrs_view = VariablesListItemView()
        self.unused_attrs_view.setModel(self.unused_attrs)
        ibox.layout().addWidget(self.unused_attrs_view)

        # Load the most recent file
        self.set_file_list()
        if len(self.recent_files) > 0:
            self.open_file(self.recent_files[0])

    def set_file_list(self):
        self.file_combo.clear()
        if not self.recent_files:
            self.file_combo.addItem("(none)")
        for file in self.recent_files:
            if file == "(none)":
                self.file_combo.addItem("(none)")
            else:
                self.file_combo.addItem(os.path.split(file)[1])
        self.file_combo.addItem("Browse documentation corpora ...")

    def reload(self):
        if self.recent_files:
            return self.open_file(self.recent_files[0])

    def select_file(self, n):
        if n < len(self.recent_files) :
            name = self.recent_files[n]
            del self.recent_files[n]
            self.recent_files.insert(0, name)
        elif n:
            self.browse_file(True)

        if len(self.recent_files) > 0:
            self.set_file_list()
            self.open_file(self.recent_files[0])

    def browse_file(self, demos_loc=False):
        start_file = os.path.expanduser("~/")
        if demos_loc:
            start_file = get_sample_corpora_dir()
        filename = QtGui.QFileDialog.getOpenFileName(
            self, 'Open Orange Document Corpus', start_file, self.dlgFormats)
        if not filename:
            return
        if filename in self.recent_files:
            self.recent_files.remove(filename)
        self.recent_files.insert(0, filename)
        self.set_file_list()
        self.open_file(filename)

    def open_file(self, path):
        self.error(1, '')
        self.used_attrs[:] = []
        self.unused_attrs[:] = []

        try:
            self.corpus = Corpus.from_file(path)
            for i in self.corpus.used_features:
                self.used_attrs.append(i)
            for i in self.corpus.domain.metas:
                if i not in self.corpus.used_features:
                    self.unused_attrs.append(i)

            self.info_label.setText("Corpus of {} documents.".format(len(self.corpus)))
            self.send(Output.CORPUS, self.corpus)
        except BaseException as err:
            self.error(1, str(err))

    def update_feature_selection(self):
        if self.corpus is not None:
            self.corpus.regenerate_documents(self.used_attrs)
            self.send(Output.CORPUS, self.corpus)